//================================================================================================
// $Id: dumpast.cpp 6308 2023-01-16 20:16:41Z aron $
//
// Project:
//		Welltype Compiler
// Contents:
//		AST dump
// Authors:
//		Aron Barath, 2014-2023
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#include "dumpast.h"
#include <iostream>
#include "wtparser/frontend.h"
#include "wtparser/astvisitor.h"
#include "wtparser/ast/ast_program.h"
#include "wtparser/ast/ast_entity.h"
#include "wtparser/ast/ast_metadata.h"
#include "wtparser/ast/ast_block.h"
#include "wtparser/ast/ast_mainblock.h"
#include "wtparser/ast/ast_constructorblock.h"
#include "wtparser/ast/ast_destructorblock.h"
#include "wtparser/ast/ast_declareblock.h"
#include "wtparser/ast/ast_compound.h"
#include "wtparser/ast/ast_exceptioncompound.h"
#include "wtparser/ast/ast_ehandler.h"
#include "wtparser/ast/ast_ehandlers.h"
#include "wtparser/ast/ast_finally.h"
#include "wtparser/ast/ast_statement.h"
#include "wtparser/ast/ast_variableaccess.h"
#include "wtparser/ast/ast_variabledeclaration.h"
#include "wtparser/ast/ast_variabledeclfragment.h"
#include "wtparser/ast/ast_elementaccess.h"
#include "wtparser/ast/ast_type.h"
#include "wtparser/ast/ast_typeinfo.h"
#include "wtparser/ast/ast_simultaneoustype.h"
#include "wtparser/ast/ast_basictype.h"
#include "wtparser/ast/ast_seqtype.h"
#include "wtparser/ast/ast_simultaneousexpression.h"
#include "wtparser/ast/ast_stringliteral.h"
#include "wtparser/ast/ast_booleanliteral.h"
#include "wtparser/ast/ast_characterliteral.h"
#include "wtparser/ast/ast_numberliteral.h"
#include "wtparser/ast/ast_assignment.h"
#include "wtparser/ast/ast_functionsignature.h"
#include "wtparser/ast/ast_functionargument.h"
#include "wtparser/ast/ast_functiondefinition.h"
#include "wtparser/ast/ast_functiondeclaration.h"
#include "wtparser/ast/ast_functioncall.h"
#include "wtparser/ast/ast_procedurecall.h"
#include "wtparser/ast/ast_recordfield.h"
#include "wtparser/ast/ast_recordtype.h"
#include "wtparser/ast/ast_recorddeclaration.h"
#include "wtparser/ast/ast_exceptiondeclaration.h"
#include "wtparser/ast/ast_precondition.h"
#include "wtparser/ast/ast_postcondition.h"
#include "wtparser/ast/ast_memberaccess.h"
#include "wtparser/ast/ast_andthen.h"
#include "wtparser/ast/ast_orelse.h"
#include "wtparser/ast/ast_returnstatement.h"
#include "wtparser/ast/ast_operatordeclaration.h"
#include "wtparser/ast/ast_raisestatement.h"
#include "wtparser/ast/ast_whilestatement.h"
#include "wtparser/ast/ast_dowhilestatement.h"
#include "wtparser/ast/ast_forstatement.h"
#include "wtparser/ast/ast_foreachstatement.h"
#include "wtparser/ast/ast_ifstatement.h"
#include "wtparser/ast/ast_globalblock.h"
#include "wtparser/ast/ast_assertstatement.h"
#include "wtparser/ast/ast_seqliteral.h"
#include "wtparser/ast/ast_recordliteral.h"
#include "wtparser/ast/ast_fieldassign.h"
#include "wtparser/ast/ast_datatype.h"
#include "wtparser/ast/ast_datatypecons.h"
#include "wtparser/ast/ast_datatypedeclaration.h"
#include "wtparser/ast/ast_datatypeliteral.h"
#include "wtparser/ast/ast_switchstatement.h"
#include "wtparser/ast/ast_datatypecasestatement.h"
#include "wtparser/ast/ast_defaultstatement.h"
#include "wtparser/ast/ast_functiontype.h"
#include "wtparser/ast/ast_functiontypedeclaration.h"
#include "wtparser/ast/ast_functionpointercall.h"
#include "wtparser/ast/ast_makefunctionpointer.h"
#include "wtparser/ast/ast_unresolvedfunctionname.h"
#include "wtparser/ast/ast_enumerator.h"
#include "wtparser/ast/ast_enumtype.h"
#include "wtparser/ast/ast_enumtypedeclaration.h"
#include "wtparser/ast/ast_enumcasestatement.h"
#include "wtparser/ast/ast_enumliteral.h"
#include "wtparser/ast/ast_recordupdate.h"

namespace wtc
{

class DumpVisitor : public wtast::AstVisitor
{
	wt_u32 mIndent;

	void Indent()
	{
		Indent(mIndent);
	}

	void Indent(wt_u32 indent)
	{
		while(indent--)
		{
			std::cout << "  ";
		}
	}

	const std::string & DecodeVarClass(wtast::AstNode::Variable::Class varclass)
	{
		switch(varclass)
		{
		case wtast::AstNode::Variable::LOCAL:
			{
				static const std::string _classname("local");
				return _classname;
			}
			break;

		case wtast::AstNode::Variable::GLOBAL:
			{
				static const std::string _classname("global");
				return _classname;
			}
			break;

		case wtast::AstNode::Variable::FORMAL:
			{
				static const std::string _classname("formal");
				return _classname;
			}
			break;

		case wtast::AstNode::Variable::READONLY:
			{
				static const std::string _classname("readonly");
				return _classname;
			}
			break;

		default:
			{
				static const std::string _classname("???");
				return _classname;
			}
		}
	}

public:
	DumpVisitor() : wtast::AstVisitor(), mIndent(0)
	{
		std::cout << std::dec;
	}

	bool Visit(wtast::Program* prog)
	{
		std::cout << "--------------------------------------------------" << std::endl;
		std::cout << "Program (" << prog->GetNumberOfEntities() << " entities)" << std::endl;
		Indent(mIndent++);
		return true;
	}

	void EndVisit(wtast::Program* prog)
	{
		Indent(--mIndent);
		std::cout << "End of Program." << std::endl;
	}

	bool Visit(wtast::MainBlock* mb)
	{
		Indent(mIndent++);
		std::cout << "MainBlock (dss=" << mb->GetDSS() << ", css=" << mb->GetCSS() << ")" << std::endl;
		return true;
	}

	void EndVisit(wtast::MainBlock*)
	{
		Indent(--mIndent);
		std::cout << "End of MainBlock." << std::endl;
	}

	bool Visit(wtast::ConstructorBlock* cb)
	{
		Indent(mIndent++);
		std::cout << "ConstructorBlock" << std::endl;
		return true;
	}

	void EndVisit(wtast::ConstructorBlock*)
	{
		Indent(--mIndent);
		std::cout << "End of ConstructorBlock." << std::endl;
	}

	bool Visit(wtast::DestructorBlock* db)
	{
		Indent(mIndent++);
		std::cout << "DestructorBlock" << std::endl;
		return true;
	}

	void EndVisit(wtast::DestructorBlock*)
	{
		Indent(--mIndent);
		std::cout << "End of DestructorBlock." << std::endl;
	}

	bool Visit(wtast::DeclareBlock* db)
	{
		Indent(mIndent++);
		std::cout << "DeclareBlock ("
			<< "import=" << db->IsImport()
			<< ", export=" << db->IsExport()
			<< ", numdecls=" << db->GetNumDeclarations()
			<< ")" << std::endl;
		return true;
	}

	void EndVisit(wtast::DeclareBlock*)
	{
		Indent(--mIndent);
		std::cout << "End of DeclareBlock." << std::endl;
	}

	bool Visit(wtast::Metadata* meta)
	{
		Indent();
		std::cout << "Metadata (key=\"" << meta->GetKey() << "\", value=\"" << meta->GetValue() << "\")" << std::endl;
		return false;
	}

	bool Visit(wtast::EHandlers* ehs)
	{
		Indent(mIndent++);
		std::cout << "EHandlers (" << ehs->GetNumberOfEHandlers() << " exception handlers)" << std::endl;
		return true;
	}

	void EndVisit(wtast::EHandlers*)
	{
		Indent(--mIndent);
		std::cout << "End of EHandlers." << std::endl;
	}

	bool Visit(wtast::EHandler* eh)
	{
		Indent(mIndent++);
		std::cout << "Exception handler: " << eh->GetName() << std::endl;
		return true;
	}

	void EndVisit(wtast::EHandler*)
	{
		Indent(--mIndent);
		std::cout << "End of EHandler." << std::endl;
	}

	bool Visit(wtast::Finally* f)
	{
		Indent(mIndent++);
		std::cout << "Finally" << std::endl;
		return true;
	}

	void EndVisit(wtast::Finally*)
	{
		Indent(--mIndent);
		std::cout << "End of Finally." << std::endl;
	}

	bool Visit(wtast::Compound* c)
	{
		Indent(mIndent++);
		std::cout << "Compound (" << c->GetNumberOfStatements() << " statements"
			<< ", name=\"" << c->GetName() << "\")" << std::endl;
		return true;
	}

	void EndVisit(wtast::Compound*)
	{
		Indent(--mIndent);
		std::cout << "End of Compound." << std::endl;
	}

	bool Visit(wtast::ExceptionCompound* ec)
	{
		Indent(mIndent++);
		std::cout << "ExceptionCompound (" << ec->GetNumberOfStatements() << " statements)" << std::endl;
		return true;
	}

	void EndVisit(wtast::ExceptionCompound*)
	{
		Indent(--mIndent);
		std::cout << "End of ExceptionCompound." << std::endl;
	}

	bool Visit(wtast::VariableDeclaration* vardecl)
	{
		Indent(mIndent++);
		std::cout << "VariableDeclaration (" << vardecl->GetNumberOfFragments() << " fragments)" << std::endl;
		return true;
	}

	void EndVisit(wtast::VariableDeclaration*)
	{
		Indent(--mIndent);
		std::cout << "End of VariableDeclaration." << std::endl;
	}

	bool Visit(wtast::VariableDeclFragment* frag)
	{
		Indent();
		std::cout << "VariableDeclFragment: Name=\"" << frag->GetName() << "\", UniqueId=" << frag->GetUniqueId() << std::endl;
		return false;
	}

	bool Visit(wtast::VariableAccess* varacc)
	{
		Indent();
		std::cout << "VariableAccess:"
			<< " Name=\"" << varacc->GetName() << "\""
			<< ", Frame=\"" << varacc->GetFrameName() << "\""
			<< ", UniqueId=" << varacc->GetUniqueId()
			<< ", VarClass=" << DecodeVarClass(varacc->GetVarClass())
			<< ", Type=" << varacc->GetType()->GetName()
			<< std::endl;
		return false;
	}

	bool Visit(wtast::ElementAccess* elemacc)
	{
		Indent(mIndent++);
		std::cout << "ElementAccess: Type=\"" << elemacc->GetType()->GetName() << "\"" << std::endl;
		return true;
	}

	void EndVisit(wtast::ElementAccess*)
	{
		Indent(--mIndent);
		std::cout << "End of ElementAccess." << std::endl;
	}

	bool Visit(wtast::BasicType* basictype)
	{
		Indent();
		std::cout << "BasicType: Name=\"" << basictype->GetName() << "\", TypeId=" << basictype->GetTypeId()
			<< ", LoadSize=" << basictype->GetLoadSize() << std::endl;
		return false;
	}

	bool Visit(wtast::SeqType* seqtype)
	{
		Indent(mIndent++);
		std::cout << "SeqType: Name=\"" << seqtype->GetName() << "\"" << std::endl;
		return true;
	}

	void EndVisit(wtast::SeqType*)
	{
		Indent(--mIndent);
		std::cout << "End of SeqType." << std::endl;
	}

	bool Visit(wtast::SimultaneousType* simtype)
	{
		Indent(mIndent++);
		std::cout << "SimultaneousType: Name=\"" << simtype->GetName() << "\", " << simtype->GetNumberOfTypes() << " types" << std::endl;
		return true;
	}

	void EndVisit(wtast::SimultaneousType*)
	{
		Indent(--mIndent);
		std::cout << "End of SimultaneousType." << std::endl;
	}

	bool Visit(wtast::RecordField* recfield)
	{
		Indent(mIndent++);
		std::cout << "RecordField: Name=\"" << recfield->GetName() << "\", Offset=" << recfield->GetOffset() << std::endl;
		return true;
	}

	void EndVisit(wtast::RecordField*)
	{
		Indent(--mIndent);
		std::cout << "End of RecordField." << std::endl;
	}

	bool Visit(wtast::RecordType* rec)
	{
		Indent(mIndent++);
		std::cout << "RecordType: Name=\"" << rec->GetName() << "\", TotalSize=" << rec->GetTotalSize()
			<< ", NumFields=" << rec->GetNumberOfFields() << ", TypeId=" << rec->GetTypeId() << std::endl;
		return true;
	}

	void EndVisit(wtast::RecordType*)
	{
		Indent(--mIndent);
		std::cout << "End of RecordType." << std::endl;
	}

	bool Visit(wtast::RecordDeclaration* recdecl)
	{
		Indent(mIndent++);
		std::cout << "RecordDeclaration: Name=\"" << recdecl->GetRecord()->GetName() << "\"" << std::endl;
		return true;
	}

	void EndVisit(wtast::RecordDeclaration*)
	{
		Indent(--mIndent);
		std::cout << "End of RecordDeclaration." << std::endl;
	}

	bool Visit(wtast::SimultaneousExpression* simexpr)
	{
		Indent(mIndent++);
		std::cout << "SimultaneousExpression: " << simexpr->GetNumberOfExpressions() << " expressions, Type=" << simexpr->GetType()->GetName() << std::endl;
		return true;
	}

	void EndVisit(wtast::SimultaneousExpression*)
	{
		Indent(--mIndent);
		std::cout << "End of SimultaneousExpression." << std::endl;
	}

	bool Visit(wtast::StringLiteral* lit)
	{
		Indent();
		std::cout << "StringLiteral: \"" << lit->GetLiteral() << "\", Type=" << lit->GetType()->GetName() << std::endl;
		return false;
	}

	bool Visit(wtast::BooleanLiteral* lit)
	{
		Indent();
		std::cout << "BooleanLiteral: " << (lit->GetLiteral()?"true":"false") << ", Type=" << lit->GetType()->GetName() << std::endl;
		return false;
	}

	bool Visit(wtast::CharacterLiteral* lit)
	{
		Indent();
		char ch = char(lit->GetLiteral());
		if(!isprint(ch)) { ch = '.'; }
		std::cout << "CharacterLiteral: \'" << ch << "\', Code=" << lit->GetLiteral() << ", Type=" << lit->GetType()->GetName() << std::endl;
		return false;
	}

	bool Visit(wtast::NumberLiteral* lit)
	{
		Indent();
		std::cout << "NumberLiteral:"
			<< " IsInteger=" << lit->IsInteger()
			<< ", IsSigned=" << lit->IsInteger()
			<< ", Is64Bit=" << lit->Is64Bit()
			<< ", Type=" << lit->GetType()->GetName()
			<< ", Value=" << lit->ToString()
			<< std::endl;
		return false;
	}

	bool Visit(wtast::Assignment* assign)
	{
		Indent(mIndent++);
		std::cout << "Assignment: " << assign->GetNumberOfLValues() << " lvalues" << std::endl;
		return true;
	}

	void EndVisit(wtast::Assignment*)
	{
		Indent(--mIndent);
		std::cout << "End of Assignment." << std::endl;
	}

	bool Visit(wtast::FunctionSignature* funcsig)
	{
		Indent(mIndent++);
		std::cout << "FunctionSignature: " << funcsig->GetName() << ", "
			<< funcsig->GetArgc() << " args, " << funcsig->GetRetc() << " rets, "
			<< (funcsig->IsPure() ? "pure" : "impure")
			<< ", UniqueId=" << funcsig->GetUniqueId()
			<< std::endl;
		return true;
	}

	void EndVisit(wtast::FunctionSignature*)
	{
		Indent(--mIndent);
		std::cout << "End of FunctionSignature." << std::endl;
	}

	bool Visit(wtast::FunctionArgument* funcarg)
	{
		Indent(mIndent++);
		std::cout << "FunctionArgument: " << funcarg->GetCanonicalName(true) << ", UniqueId=" << funcarg->GetUniqueId() << std::endl;
		return true;
	}

	void EndVisit(wtast::FunctionArgument*)
	{
		Indent(--mIndent);
		std::cout << "End of FunctionArgument." << std::endl;
	}

	bool Visit(wtast::FunctionDefinition* funcdef)
	{
		Indent(mIndent++);
		std::cout << "FunctionDefinition: " << funcdef->GetFunctionSignature()->GetCanonicalName(true, true) << std::endl;
		return true;
	}

	void EndVisit(wtast::FunctionDefinition*)
	{
		Indent(--mIndent);
		std::cout << "End of FunctionDefinition." << std::endl;
	}

	bool Visit(wtast::FunctionDeclaration* funcdecl)
	{
		Indent(mIndent++);
		std::cout << "FunctionDeclaration: " << funcdecl->GetFunctionSignature()->GetCanonicalName(true, true) << std::endl;
		return true;
	}

	void EndVisit(wtast::FunctionDeclaration*)
	{
		Indent(--mIndent);
		std::cout << "End of FunctionDeclaration." << std::endl;
	}

	bool Visit(wtast::FunctionCall* funccall)
	{
		Indent(mIndent++);
		std::cout << "FunctionCall: " << funccall->GetFunctionSignature()->GetCanonicalName(true, true) << std::endl;
		return true;
	}

	void EndVisit(wtast::FunctionCall*)
	{
		Indent(--mIndent);
		std::cout << "End of FunctionCall." << std::endl;
	}

	bool Visit(wtast::ProcedureCall*)
	{
		Indent(mIndent++);
		std::cout << "ProcedureCall" << std::endl;
		return true;
	}

	void EndVisit(wtast::ProcedureCall*)
	{
		Indent(--mIndent);
		std::cout << "End of ProcedureCall." << std::endl;
	}

	bool Visit(wtast::ExceptionDeclaration* excdecl)
	{
		Indent();
		std::cout << "ExceptionDeclaration: " << excdecl->GetName() << ", UniqueId=" << excdecl->GetUniqueId() << std::endl;
		return false;
	}

	bool Visit(wtast::OperatorDeclaration* opdecl)
	{
		Indent();
		std::cout << "OperatorDeclaration: " << opdecl->GetOpName() << ", Assoc=" << opdecl->GetAssociativity() << ", Prec=" << opdecl->GetPrecedence() << std::endl;
		return false;
	}

	bool Visit(wtast::RaiseStatement* raisestmt)
	{
		Indent();
		std::cout << "RaiseStatement: " << raisestmt->GetException() << ", UniqueId=" << raisestmt->GetUniqueId() << ", Near=" << raisestmt->IsNear() << std::endl;
		return false;
	}

	bool Visit(wtast::UnresolvedFunctionName* ufn)
	{
		Indent();
		std::cout << "UnresolvedFunctionName: \"" << ufn->GetName() << '\"' << std::endl;
		return false;
	}

	bool Visit(wtast::PreCondition*)
	{
		Indent(mIndent++);
		std::cout << "PreCondition" << std::endl;
		return true;
	}

	void EndVisit(wtast::PreCondition*)
	{
		Indent(--mIndent);
		std::cout << "End of PreCondition." << std::endl;
	}

	bool Visit(wtast::PostCondition* postcond)
	{
		Indent(mIndent++);
		std::cout << "PostCondition, "
			<< postcond->GetNumReturnValues() << " return values, "
			<< postcond->GetNumOldValues() << " old values, "
			<< std::endl;
		return true;
	}

	void EndVisit(wtast::PostCondition*)
	{
		Indent(--mIndent);
		std::cout << "End of PostCondition." << std::endl;
	}

	bool Visit(wtast::MemberAccess* memberaccess)
	{
		Indent(mIndent++);
		std::cout << "MemberAccess, Field=\"" << memberaccess->GetFieldName()
			<< "\", Type=" << memberaccess->GetType()->GetName()
			<< ", Offset=" << memberaccess->GetOffset() << std::endl;
		return true;
	}

	void EndVisit(wtast::MemberAccess*)
	{
		Indent(--mIndent);
		std::cout << "End of MemberAccess." << std::endl;
	}

	bool Visit(wtast::AndThen* andthen)
	{
		Indent(mIndent++);
		std::cout << "AndThen, Type=" << andthen->GetType()->GetName() << std::endl;
		return true;
	}

	void EndVisit(wtast::AndThen*)
	{
		Indent(--mIndent);
		std::cout << "End of AndThen." << std::endl;
	}

	bool Visit(wtast::OrElse* orelse)
	{
		Indent(mIndent++);
		std::cout << "OrElse, Type=" << orelse->GetType()->GetName() << std::endl;
		return true;
	}

	void EndVisit(wtast::OrElse*)
	{
		Indent(--mIndent);
		std::cout << "End of OrElse." << std::endl;
	}

	bool Visit(wtast::ReturnStatement*)
	{
		Indent(mIndent++);
		std::cout << "ReturnStatement" << std::endl;
		return true;
	}

	void EndVisit(wtast::ReturnStatement*)
	{
		Indent(--mIndent);
		std::cout << "End of ReturnStatement." << std::endl;
	}

	bool Visit(wtast::WhileStatement*)
	{
		Indent(mIndent++);
		std::cout << "WhileStatement" << std::endl;
		return true;
	}

	void EndVisit(wtast::WhileStatement*)
	{
		Indent(--mIndent);
		std::cout << "End of WhileStatement." << std::endl;
	}

	bool Visit(wtast::DoWhileStatement*)
	{
		Indent(mIndent++);
		std::cout << "DoWhileStatement" << std::endl;
		return true;
	}

	void EndVisit(wtast::DoWhileStatement*)
	{
		Indent(--mIndent);
		std::cout << "End of DoWhileStatement." << std::endl;
	}

	bool Visit(wtast::ForStatement*)
	{
		Indent(mIndent++);
		std::cout << "ForStatement" << std::endl;
		return true;
	}

	void EndVisit(wtast::ForStatement*)
	{
		Indent(--mIndent);
		std::cout << "End of ForStatement." << std::endl;
	}

	bool Visit(wtast::ForeachStatement* foreach)
	{
		Indent(mIndent++);
		std::cout << "ForeachStatement, "
			<< (foreach->IsInclusive()?"inclusive":"exclusive") << ", "
			<< (foreach->IsReverse()?"reverse":"normal")
			<< std::endl;
		return true;
	}

	void EndVisit(wtast::ForeachStatement*)
	{
		Indent(--mIndent);
		std::cout << "End of ForeachStatement." << std::endl;
	}

	bool Visit(wtast::IfStatement*)
	{
		Indent(mIndent++);
		std::cout << "IfStatement" << std::endl;
		return true;
	}

	void EndVisit(wtast::IfStatement*)
	{
		Indent(--mIndent);
		std::cout << "End of IfStatement." << std::endl;
	}

	bool Visit(wtast::GlobalBlock* gb)
	{
		Indent(mIndent++);
		std::cout << "GlobalBlock, " << gb->GetNumVariableDeclarations() << " variable declarations" << std::endl;
		return true;
	}

	void EndVisit(wtast::GlobalBlock*)
	{
		Indent(--mIndent);
		std::cout << "End of GlobalBlock." << std::endl;
	}

	bool Visit(wtast::AssertStatement*)
	{
		Indent(mIndent++);
		std::cout << "AssertStatement" << std::endl;
		return true;
	}

	void EndVisit(wtast::AssertStatement*)
	{
		Indent(--mIndent);
		std::cout << "End of AssertStatement." << std::endl;
	}

	bool Visit(wtast::SeqLiteral* sl)
	{
		Indent(mIndent++);
		std::cout << "SeqLiteral (type=" << sl->GetType()->GetName() << ")" << std::endl;
		return true;
	}

	void EndVisit(wtast::SeqLiteral*)
	{
		Indent(--mIndent);
		std::cout << "End of SeqLiteral." << std::endl;
	}

	bool Visit(wtast::RecordLiteral* rl)
	{
		Indent(mIndent++);
		std::cout << "RecordLiteral (type=" << rl->GetType()->GetName() << ")" << std::endl;
		return true;
	}

	void EndVisit(wtast::RecordLiteral*)
	{
		Indent(--mIndent);
		std::cout << "End of RecordLiteral." << std::endl;
	}

	bool Visit(wtast::FieldAssign* fa)
	{
		Indent(mIndent++);
		std::cout << "FieldAssign (" << fa->GetNumberOfFieldNames() << " names: ";
		for(std::size_t i=0,n=fa->GetNumberOfFieldNames();i!=n;++i)
		{
			if(i) { std::cout << ", "; }
			std::cout << fa->GetFieldName(i);
		}
		std::cout << ')' << std::endl;
		return true;
	}

	void EndVisit(wtast::FieldAssign*)
	{
		Indent(--mIndent);
		std::cout << "End of FieldAssign." << std::endl;
	}

	bool Visit(wtast::DataType* dt)
	{
		Indent(mIndent++);
		std::cout << "DataType, Name: \"" << dt->GetName() << "\", TypeId=" << dt->GetTypeId() << std::endl;
		return true;
	}

	void EndVisit(wtast::DataType*)
	{
		Indent(--mIndent);
		std::cout << "End of DataType." << std::endl;
	}

	bool Visit(wtast::DataTypeCons* dtc)
	{
		Indent(mIndent++);
		std::cout << "DataTypeCons, Name: \"" << dtc->GetName() << "\", Id: " << dtc->GetConsId() << ", Size: " << dtc->GetDataSize() << std::endl;
		return true;
	}

	void EndVisit(wtast::DataTypeCons*)
	{
		Indent(--mIndent);
		std::cout << "End of DataTypeCons." << std::endl;
	}

	bool Visit(wtast::DataTypeDeclaration* dtd)
	{
		Indent(mIndent++);
		std::cout << "DataTypeDeclaration, Name: \"" << dtd->GetDataType()->GetName() << "\"" << std::endl;
		return true;
	}

	void EndVisit(wtast::DataTypeDeclaration*)
	{
		Indent(--mIndent);
		std::cout << "End of DataTypeDeclaration." << std::endl;
	}

	bool Visit(wtast::DataTypeLiteral* dtl)
	{
		Indent(mIndent++);
		std::cout << "DataTypeLiteral, DataType: \"" << dtl->GetDataType()->GetName()
			<< "\", Cons: \"" << dtl->GetCons()->GetName() << "\"" << std::endl;
		return true;
	}

	void EndVisit(wtast::DataTypeLiteral*)
	{
		Indent(--mIndent);
		std::cout << "End of DataTypeLiteral." << std::endl;
	}

	bool Visit(wtast::SwitchStatement* sw)
	{
		Indent(mIndent++);
		std::cout << "SwitchStatement, Type: \"" << sw->GetCondition()->GetType()->GetName() << "\"" << std::endl;
		return true;
	}

	void EndVisit(wtast::SwitchStatement*)
	{
		Indent(--mIndent);
		std::cout << "End of SwitchStatement." << std::endl;
	}

	bool Visit(wtast::DataTypeCaseStatement* dtcs)
	{
		Indent(mIndent++);
		std::cout << "DataTypeCaseStatement, Cons: \"" << dtcs->GetCons()->GetName() << "\"" << std::endl;
		return true;
	}

	void EndVisit(wtast::DataTypeCaseStatement*)
	{
		Indent(--mIndent);
		std::cout << "End of DataTypeCaseStatement." << std::endl;
	}

	bool Visit(wtast::DefaultStatement* cas)
	{
		Indent(mIndent++);
		std::cout << "DefaultStatement" << std::endl;
		return true;
	}

	void EndVisit(wtast::DefaultStatement*)
	{
		Indent(--mIndent);
		std::cout << "End of DefaultStatement." << std::endl;
	}

	bool Visit(wtast::FunctionType* ft)
	{
		Indent(mIndent++);
		std::cout << "FunctionType, Name: \"" << ft->GetName() << "\", TypeId=" << ft->GetTypeId() << std::endl;
		return true;
	}

	void EndVisit(wtast::FunctionType*)
	{
		Indent(--mIndent);
		std::cout << "End of FunctionType." << std::endl;
	}

	bool Visit(wtast::FunctionTypeDeclaration* ftd)
	{
		Indent(mIndent++);
		std::cout << "FunctionTypeDeclaration, Name: \"" << ftd->GetFunctionType()->GetName() << "\"" << std::endl;
		return true;
	}

	void EndVisit(wtast::FunctionTypeDeclaration*)
	{
		Indent(--mIndent);
		std::cout << "End of FunctionTypeDeclaration." << std::endl;
	}

	bool Visit(wtast::FunctionPointerCall* funcptrcall)
	{
		Indent(mIndent++);
		std::cout << "FunctionPointerCall: " << funcptrcall->GetFunctionSignature()->GetCanonicalName(true, true) << std::endl;
		return true;
	}

	void EndVisit(wtast::FunctionPointerCall*)
	{
		Indent(--mIndent);
		std::cout << "End of FunctionPointerCall." << std::endl;
	}

	bool Visit(wtast::MakeFunctionPointer* makefuncptr)
	{
		Indent(mIndent++);
		std::cout << "MakeFunctionPointer: " << makefuncptr->GetFunctionSignature()->GetCanonicalName(true, true)
			<< ", Type: " << makefuncptr->GetType()->GetName() << std::endl;
		return true;
	}

	void EndVisit(wtast::MakeFunctionPointer*)
	{
		Indent(--mIndent);
		std::cout << "End of MakeFunctionPointer." << std::endl;
	}

	bool Visit(wtast::Enumerator* e)
	{
		Indent(mIndent);
		std::cout << "Enumerator, Name: \"" << e->GetName() << "\", Ordinal: " << e->GetOrdinal() << std::endl;
		return true;
	}

	bool Visit(wtast::EnumType* et)
	{
		Indent(mIndent++);
		std::cout << "EnumType, Name: \"" << et->GetName() << "\", TypeId: " << et->GetTypeId() << std::endl;
		return true;
	}

	void EndVisit(wtast::EnumType*)
	{
		Indent(--mIndent);
		std::cout << "End of EnumType." << std::endl;
	}

	bool Visit(wtast::EnumTypeDeclaration* etd)
	{
		Indent(mIndent++);
		std::cout << "EnumTypeDeclaration, Name: \"" << etd->GetEnumType()->GetName() << "\"" << std::endl;
		return true;
	}

	void EndVisit(wtast::EnumTypeDeclaration*)
	{
		Indent(--mIndent);
		std::cout << "End of EnumTypeDeclaration." << std::endl;
	}

	bool Visit(wtast::EnumCaseStatement* ecs)
	{
		Indent(mIndent++);
		std::cout << "EnumCaseStatement, Enumerator: \"" << ecs->GetEnumerator()->GetName() << "\"" << std::endl;
		return true;
	}

	void EndVisit(wtast::EnumCaseStatement*)
	{
		Indent(--mIndent);
		std::cout << "End of EnumCaseStatement." << std::endl;
	}

	bool Visit(wtast::EnumLiteral* el)
	{
		Indent(mIndent);
		std::cout << "EnumLiteral, Enumerator: \"" << el->GetEnumType()->GetName() << "::" << el->GetEnumerator()->GetName() << "\"" << std::endl;
		return false;
	}

	bool Visit(wtast::RecordUpdate* ru)
	{
		Indent(mIndent++);
		std::cout << "RecordUpdate" << std::endl;
		return true;
	}

	void EndVisit(wtast::RecordUpdate*)
	{
		Indent(--mIndent);
		std::cout << "End of RecordUpdate." << std::endl;
	}
};

void DumpAST(wtast::Program* program)
{
	DumpVisitor dumpvisitor;

	program->Accept(&dumpvisitor);
}

} // wtc
