/* $Id: ast_functionargument.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_FUNCTIONARGUMENT_H
#define WTPARSER_AST_FUNCTIONARGUMENT_H

#include <string>
#include "../astnode.h"
#include "../astvisitor.h"
#include "../uniqueid.h"

namespace wtast
{

class Type;

class FunctionArgument : public AstNode
{
	Type*              mType;
	std::string        mName;
	wtparser::UniqueId mUniqueId;

public:
	FunctionArgument(Type* type, const std::string & name)
		: AstNode()
		, mType(type)
		, mName(name)
		, mUniqueId(0)
	{
		// ok
	}

	~FunctionArgument();

	void Accept(AstVisitor* vis);

	FunctionArgument* Copy();

	Kind GetKind() const
	{
		return FUNCTIONARGUMENT;
	}

	Type* GetType()
	{
		return mType;
	}

	std::string & GetName()
	{
		return mName;
	}

	std::string GetCanonicalName(bool argnames);

	wtparser::UniqueId GetUniqueId() const
	{
		return mUniqueId;
	}

	void SetUniqueId(wtparser::UniqueId unique_id)
	{
		mUniqueId = unique_id;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_FUNCTIONARGUMENT_H

