/* $Id: ast_makefunctionpointer.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_MAKEFUNCTIONPOINTER_H
#define WTPARSER_AST_MAKEFUNCTIONPOINTER_H

#include "../astnode.h"
#include "ast_expression.h"
#include "ast_typeinfo.h"
#include <string>

namespace wtast
{

class FunctionType;
class FunctionSignature;

class MakeFunctionPointer : public Expression
{
	FunctionType*      mFuncType;
	FunctionSignature* mFuncSig;

public:
	MakeFunctionPointer(FunctionType* functype, FunctionSignature* funcsig)
		: Expression()
		, mFuncType(functype)
		, mFuncSig(funcsig)
	{
	}

	~MakeFunctionPointer();

	void Accept(AstVisitor* vis);

	MakeFunctionPointer* Copy();

	Kind GetKind() const
	{
		return MAKEFUNCTIONPOINTER;
	}

	bool OriginateFrom(Variable::Class varclass)
	{
		return Variable::READONLY==varclass || Variable::LOCAL==varclass;
	}

	TypeInfo* GetType()
	{
		return reinterpret_cast<TypeInfo*>(mFuncType);
	}

	FunctionSignature* GetFunctionSignature()
	{
		return mFuncSig;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_MAKEFUNCTIONPOINTER_H
