/* $Id: ast_seqliteral.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_SEQLITERAL_H
#define WTPARSER_AST_SEQLITERAL_H

#include "../astnode.h"
#include "ast_expression.h"
#include "ast_seqtype.h"
#include <string>

namespace wtast
{

class Type;

class SeqLiteral : public Expression
{
	SeqType*    mType;
	Expression* mInitializer;

public:
	SeqLiteral(SeqType* type, Expression* init)
		: Expression()
		, mType(type)
		, mInitializer(init)
	{
		// ok
	}

	~SeqLiteral();

	void Accept(AstVisitor* vis);

	SeqLiteral* Copy();

	Kind GetKind() const
	{
		return SEQLITERAL;
	}

	bool OriginateFrom(Variable::Class varclass);

	Expression* GetInitializer()
	{
		return mInitializer;
	}

	SeqType* GetSeqType()
	{
		return mType;
	}

	Type* GetElementType();

	TypeInfo* GetType();

	virtual bool IsLiteral() const { return true; } // from wtast::Expression
	virtual bool IsTemporary() const { return true; } // from wtast::Expression
};

} // namespace wtast

#endif // WTPARSER_AST_SEQLITERAL_H
