//================================================================================================
// $Id: builtins.cpp 6463 2023-02-27 19:54:59Z aron $
//
// Project:
//		Welltype Parser
// Contents:
//		Builtin functions
// Authors:
//		Aron Barath, 2014-2023
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#include "builtins.h"
#include "wtruntime/stdlib.h"

namespace wtparser
{
namespace builtin
{

namespace
{
	TypeVec MkTV() { TypeVec tv; return tv; }
	TypeVec MkTV(wt_u32 t1) { TypeVec tv; tv.push_back(t1); return tv; }
	TypeVec MkTV(wt_u32 t1, wt_u32 t2) { TypeVec tv; tv.push_back(t1); tv.push_back(t2); return tv; }
	TypeVec MkTV(wt_u32 t1, wt_u32 t2, wt_u32 t3) { TypeVec tv; tv.push_back(t1); tv.push_back(t2); tv.push_back(t3); return tv; }
//	TypeVec MkTV(wt_u32 t1, wt_u32 t2, wt_u32 t3, wt_u32 t4) { TypeVec tv; tv.push_back(t1); tv.push_back(t2); tv.push_back(t3); tv.push_back(t4); return tv; }
	TypeVec MkTV(wt_u32 t1, wt_u32 t2, wt_u32 t3, wt_u32 t4, wt_u32 t5) { TypeVec tv; tv.push_back(t1); tv.push_back(t2); tv.push_back(t3); tv.push_back(t4); tv.push_back(t5); return tv; }
//	TypeVec MkTV(wt_u32 t1, wt_u32 t2, wt_u32 t3, wt_u32 t4, wt_u32 t5, wt_u32 t6) { TypeVec tv; tv.push_back(t1); tv.push_back(t2); tv.push_back(t3); tv.push_back(t4); tv.push_back(t5); tv.push_back(t6); return tv; }
//	TypeVec MkTV(wt_u32 t1, wt_u32 t2, wt_u32 t3, wt_u32 t4, wt_u32 t5, wt_u32 t6, wt_u32 t7) { TypeVec tv; tv.push_back(t1); tv.push_back(t2); tv.push_back(t3); tv.push_back(t4); tv.push_back(t5); tv.push_back(t6); tv.push_back(t7); return tv; }
//	TypeVec MkTV(wt_u32 t1, wt_u32 t2, wt_u32 t3, wt_u32 t4, wt_u32 t5, wt_u32 t6, wt_u32 t7, wt_u32 t8) { TypeVec tv; tv.push_back(t1); tv.push_back(t2); tv.push_back(t3); tv.push_back(t4); tv.push_back(t5); tv.push_back(t6); tv.push_back(t7); tv.push_back(t8); return tv; }
//	TypeVec MkTV(wt_u32 t1, wt_u32 t2, wt_u32 t3, wt_u32 t4, wt_u32 t5, wt_u32 t6, wt_u32 t7, wt_u32 t8, wt_u32 t9) { TypeVec tv; tv.push_back(t1); tv.push_back(t2); tv.push_back(t3); tv.push_back(t4); tv.push_back(t5); tv.push_back(t6); tv.push_back(t7); tv.push_back(t8); tv.push_back(t9); return tv; }
	TypeVec MkTV(wt_u32 t1, wt_u32 t2, wt_u32 t3, wt_u32 t4, wt_u32 t5, wt_u32 t6, wt_u32 t7, wt_u32 t8, wt_u32 t9, wt_u32 t10) { TypeVec tv; tv.push_back(t1); tv.push_back(t2); tv.push_back(t3); tv.push_back(t4); tv.push_back(t5); tv.push_back(t6); tv.push_back(t7); tv.push_back(t8); tv.push_back(t9); tv.push_back(t10); return tv; }

	InstructionVec MkIV() { InstructionVec iv; return iv; }
	InstructionVec MkIV(wt_u16 i1) { InstructionVec iv; iv.push_back(i1); return iv; }
	InstructionVec MkIV(wt_u16 i1, wt_u16 i2) { InstructionVec iv; iv.push_back(i1); iv.push_back(i2); return iv; }
//	InstructionVec MkIV(wt_u16 i1, wt_u16 i2, wt_u16 i3) { InstructionVec iv; iv.push_back(i1); iv.push_back(i2); iv.push_back(i3); return iv; }
}

Builtins::Builtins() : mLastUniqueId(0)
{
	// console

	mBuiltins.push_back(Builtin("write",
		MkTV(WTT_STRING),
		MkTV(),
		false,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CONWRITE)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("write",
		MkTV(WTT_CHAR),
		MkTV(),
		false,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CONWRITECH)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("read",
		MkTV(),
		MkTV(WTT_STRING),
		false,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_PWZ_I, '\n'), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CONREADLN)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("read",
		MkTV(WTT_CHAR),
		MkTV(WTT_STRING),
		false,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CONREADLN)),
		++mLastUniqueId));

	// version

	mBuiltins.push_back(Builtin("wt_version",
		MkTV(),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_VERSION)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("wt_compversion",
		MkTV(),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_COMPVERSION)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("wt_coreversion",
		MkTV(),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_COREVERSION)),
		++mLastUniqueId));

	// clz, ctz

	mBuiltins.push_back(Builtin("clz",
		MkTV(WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CLZ8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("clz",
		MkTV(WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CLZ16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("clz",
		MkTV(WTT_U32),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CLZ32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("clz",
		MkTV(WTT_U64),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CLZ64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ctz",
		MkTV(WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CTZ8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ctz",
		MkTV(WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CTZ16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ctz",
		MkTV(WTT_U32),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CTZ32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ctz",
		MkTV(WTT_U64),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CTZ64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("clz",
		MkTV(WTT_I8),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CLZ8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("clz",
		MkTV(WTT_I16),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CLZ16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("clz",
		MkTV(WTT_I32),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CLZ32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("clz",
		MkTV(WTT_I64),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CLZ64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ctz",
		MkTV(WTT_I8),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CTZ8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ctz",
		MkTV(WTT_I16),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CTZ16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ctz",
		MkTV(WTT_I32),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CTZ32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ctz",
		MkTV(WTT_I64),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CTZ64)),
		++mLastUniqueId));

	// ror

	mBuiltins.push_back(Builtin("ror",
		MkTV(WTT_I8, WTT_I8),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_RORB, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ror",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_RORB, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ror",
		MkTV(WTT_I16, WTT_I16),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_RORH, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ror",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_RORH, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ror",
		MkTV(WTT_I32, WTT_I32),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_ROR_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ror",
		MkTV(WTT_U32, WTT_U32),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_ROR_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ror",
		MkTV(WTT_I64, WTT_I64),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_RORL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ror",
		MkTV(WTT_U64, WTT_U64),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_RORL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ror",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_RORH, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ror",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_RORB, 0)),
		++mLastUniqueId));

	// sqrt

	mBuiltins.push_back(Builtin("sqrt",
		MkTV(WTT_I32),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_I32SQRT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("sqrt",
		MkTV(WTT_U32),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_U32SQRT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("sqrt",
		MkTV(WTT_I64),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_I64SQRT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("sqrt",
		MkTV(WTT_U64),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_U64SQRT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("sqrt",
		MkTV(WTT_I16),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_I16SQRT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("sqrt",
		MkTV(WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_U16SQRT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("sqrt",
		MkTV(WTT_I8),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_I8SQRT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("sqrt",
		MkTV(WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_U8SQRT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("sqrt",
		MkTV(WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F32SQRT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("sqrt",
		MkTV(WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F64SQRT)),
		++mLastUniqueId));

	// float math

	mBuiltins.push_back(Builtin("log",
		MkTV(WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F32LOG)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("log",
		MkTV(WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F64LOG)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("exp",
		MkTV(WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F32EXP)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("exp",
		MkTV(WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F64EXP)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("sin",
		MkTV(WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F32SIN)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("sin",
		MkTV(WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F64SIN)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("cos",
		MkTV(WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F32COS)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("cos",
		MkTV(WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F64COS)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tan",
		MkTV(WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F32TAN)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tan",
		MkTV(WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F64TAN)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("asin",
		MkTV(WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F32ASIN)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("asin",
		MkTV(WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F64ASIN)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("acos",
		MkTV(WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F32ACOS)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("acos",
		MkTV(WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F64ACOS)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("atan",
		MkTV(WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F32ATAN)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("atan",
		MkTV(WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F64ATAN)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("atan2",
		MkTV(WTT_F32, WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F32ATAN2)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("atan2",
		MkTV(WTT_F64, WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F64ATAN2)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("pow",
		MkTV(WTT_F32, WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F32POW)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("pow",
		MkTV(WTT_F64, WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_F64POW)),
		++mLastUniqueId));

	// string and char

	mBuiltins.push_back(Builtin("strcmp",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCMP)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("stricmp",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRICMP)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("streq",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strieq",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRICMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strne",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strine",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRICMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<=",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">=",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("trim",
		MkTV(WTT_STRING),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRTRIM)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("trim",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRTRIM2)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("empty",
		MkTV(WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STREMPTY)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("startswith",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRSTARTSWITH)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("endswith",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRENDSWITH)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("chars",
		MkTV(WTT_CHAR, WTT_U32),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCHARS)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("chars",
		MkTV(WTT_CHAR, WTT_U32, WTT_STRING),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCHARS2)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("copies",
		MkTV(WTT_STRING, WTT_U32),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCOPIES)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("copies",
		MkTV(WTT_STRING, WTT_U32, WTT_STRING),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCOPIES2)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strcpy",
		MkTV(WTTF_MUTABLE|WTT_STRING, WTT_U32, WTT_STRING, WTT_U32, WTT_U32),
		MkTV(),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCPY)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strstr",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRSTR)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strstr",
		MkTV(WTT_STRING, WTT_U32, WTT_STRING),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRSTR2)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strrstr",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRRSTR)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strrstr",
		MkTV(WTT_STRING, WTT_U32, WTT_STRING),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRRSTR2)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strchr",
		MkTV(WTT_STRING, WTT_CHAR),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCHR)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strchr",
		MkTV(WTT_STRING, WTT_U32, WTT_CHAR),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCHR2)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strrchr",
		MkTV(WTT_STRING, WTT_CHAR),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRRCHR)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strrchr",
		MkTV(WTT_STRING, WTT_U32, WTT_CHAR),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRRCHR2)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strspn",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRSPN)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strspn",
		MkTV(WTT_STRING, WTT_U32, WTT_STRING),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRSPN2)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strcspn",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCSPN)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strcspn",
		MkTV(WTT_STRING, WTT_U32, WTT_STRING),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCSPN2)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strtok",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTTF_SEQ | WTT_STRING),
		true, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fromutf8",
		MkTV(WTT_STRING),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRFROMUTF8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toutf8",
		MkTV(WTT_STRING),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRTOUTF8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strcat",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCAT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_STRING, WTT_STRING),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRCAT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strrev",
		MkTV(WTT_STRING),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRREV)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("substr",
		MkTV(WTT_STRING, WTT_U32, WTT_U32),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRSUB)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("substr",
		MkTV(WTT_STRING, WTT_U32),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRSUB2)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strlen",
		MkTV(WTT_STRING),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_STRL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strupper",
		MkTV(WTT_STRING),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRUPPER)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("strlower",
		MkTV(WTT_STRING),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRLOWER)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("chartostring",
		MkTV(WTT_CHAR),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARTOSTRING)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("getstrchar",
		MkTV(WTT_STRING, WTT_U32),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_STRCR_S, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_LDB, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("charcmp",
		MkTV(WTT_CHAR, WTT_CHAR),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARCMP)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("charadd",
		MkTV(WTT_CHAR, WTT_CHAR),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARADD)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("charsub",
		MkTV(WTT_CHAR, WTT_CHAR),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARSUB)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("charinc",
		MkTV(WTT_CHAR, WTT_I32),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARINC)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("chardiff",
		MkTV(WTT_CHAR, WTT_CHAR),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARDIFF)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tolower",
		MkTV(WTT_CHAR),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARTOLOWER)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toupper",
		MkTV(WTT_CHAR),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARTOUPPER)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_CHAR, WTT_I8),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARINC)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_CHAR, WTT_U8),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARINC)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_CHAR, WTT_I16),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARINC)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_CHAR, WTT_U16),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARINC)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_CHAR, WTT_I32),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARINC)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_CHAR, WTT_U32),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARINC)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_CHAR, WTT_CHAR),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARDIFF)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_CHAR, WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_CHAR, WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<",
		MkTV(WTT_CHAR, WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<=",
		MkTV(WTT_CHAR, WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">",
		MkTV(WTT_CHAR, WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">=",
		MkTV(WTT_CHAR, WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARCMP), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGE, 0)),
		++mLastUniqueId));

	// conversion

	mBuiltins.push_back(Builtin("tos",
		MkTV(WTT_STRING),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_STRDUP)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tos",
		MkTV(WTT_CHAR),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CHARTOSTRING)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tos",
		MkTV(WTT_BOOL),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CBOOLSTRING)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tos",
		MkTV(WTT_I8),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 8), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32STRING)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tos",
		MkTV(WTT_U8),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32STRING)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tos",
		MkTV(WTT_I16),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 16), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32STRING)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tos",
		MkTV(WTT_U16),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32STRING)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tos",
		MkTV(WTT_I32),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32STRING)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tos",
		MkTV(WTT_U32),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32STRING)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tos",
		MkTV(WTT_I64),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI64STRING)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tos",
		MkTV(WTT_U64),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU64STRING)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tos",
		MkTV(WTT_F32),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CF32STRING)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tos",
		MkTV(WTT_F64),
		MkTV(WTT_STRING),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CF64STRING)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tobool",
		MkTV(WTT_STRING),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CSTRINGBOOL)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi32",
		MkTV(WTT_STRING),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CSTRINGI32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi32",
		MkTV(WTT_I8),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi32",
		MkTV(WTT_U8),
		MkTV(WTT_I32),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi32",
		MkTV(WTT_I16),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi32",
		MkTV(WTT_U16),
		MkTV(WTT_I32),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi32",
		MkTV(WTT_I32),
		MkTV(WTT_I32),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi32",
		MkTV(WTT_U32),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32I32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi32",
		MkTV(WTT_I64),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI64I32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi32",
		MkTV(WTT_U64),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU64I32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi32",
		MkTV(WTT_F32),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CF32I32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi32",
		MkTV(WTT_F64),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CF64I32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou32",
		MkTV(WTT_STRING),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CSTRINGU32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou32",
		MkTV(WTT_I8),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 8), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32U32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou32",
		MkTV(WTT_U8),
		MkTV(WTT_U32),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou32",
		MkTV(WTT_I16),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 16), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32U32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou32",
		MkTV(WTT_U16),
		MkTV(WTT_U32),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou32",
		MkTV(WTT_I32),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32U32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou32",
		MkTV(WTT_U32),
		MkTV(WTT_U32),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou32",
		MkTV(WTT_I64),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI64U32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou32",
		MkTV(WTT_U64),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU64U32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou32",
		MkTV(WTT_F32),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CF32U32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou32",
		MkTV(WTT_F64),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CF64U32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi64",
		MkTV(WTT_STRING),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CSTRINGI64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi64",
		MkTV(WTT_I8),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 8), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32I64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi64",
		MkTV(WTT_U8),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32I64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi64",
		MkTV(WTT_I16),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 16), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32I64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi64",
		MkTV(WTT_U16),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32I64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi64",
		MkTV(WTT_I32),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32I64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi64",
		MkTV(WTT_U32),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32I64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi64",
		MkTV(WTT_I64),
		MkTV(WTT_I64),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi64",
		MkTV(WTT_U64),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU64I64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi64",
		MkTV(WTT_F32),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CF32I64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi64",
		MkTV(WTT_F64),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CF64I64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou64",
		MkTV(WTT_STRING),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CSTRINGU64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou64",
		MkTV(WTT_I8),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 8), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32U64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou64",
		MkTV(WTT_U8),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32U64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou64",
		MkTV(WTT_I16),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 16), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32U64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou64",
		MkTV(WTT_U16),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32U64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou64",
		MkTV(WTT_I32),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32U64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou64",
		MkTV(WTT_U32),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32U64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou64",
		MkTV(WTT_I64),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI64U64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou64",
		MkTV(WTT_U64),
		MkTV(WTT_U64),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou64",
		MkTV(WTT_F32),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CF32U64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou64",
		MkTV(WTT_F64),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CF64U64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof32",
		MkTV(WTT_STRING),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CSTRINGF32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof32",
		MkTV(WTT_I8),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 8), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32F32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof32",
		MkTV(WTT_U8),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32F32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof32",
		MkTV(WTT_I16),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 16), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32F32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof32",
		MkTV(WTT_U16),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32F32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof32",
		MkTV(WTT_I32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32F32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof32",
		MkTV(WTT_U32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32F32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof32",
		MkTV(WTT_I64),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI64F32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof32",
		MkTV(WTT_U64),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU64F32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof32",
		MkTV(WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof32",
		MkTV(WTT_F64),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CF64F32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof64",
		MkTV(WTT_STRING),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CSTRINGF64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof64",
		MkTV(WTT_I8),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 8), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32F64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof64",
		MkTV(WTT_U8),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32F64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof64",
		MkTV(WTT_I16),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 16), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32F64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof64",
		MkTV(WTT_U16),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32F64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof64",
		MkTV(WTT_I32),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32F64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof64",
		MkTV(WTT_U32),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32F64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof64",
		MkTV(WTT_I64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI64F64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof64",
		MkTV(WTT_U64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU64F64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof64",
		MkTV(WTT_F32),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CF32F64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tof64",
		MkTV(WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tochar",
		MkTV(WTT_CHAR),
		MkTV(WTT_CHAR),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tochar",
		MkTV(WTT_I8),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 8), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32CHAR)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tochar",
		MkTV(WTT_U8),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32CHAR)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tochar",
		MkTV(WTT_I16),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 16), WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32CHAR)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tochar",
		MkTV(WTT_U16),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32CHAR)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tochar",
		MkTV(WTT_I32),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32CHAR)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tochar",
		MkTV(WTT_U32),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32CHAR)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tochar",
		MkTV(WTT_I64),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI64CHAR)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tochar",
		MkTV(WTT_U64),
		MkTV(WTT_CHAR),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU64CHAR)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi8",
		MkTV(WTT_CHAR),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CCHARI32), WT_TOMB_BUILDINSTR(WT_TOMB_OP_MSKBTM, 8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou8",
		MkTV(WTT_CHAR),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CCHARU32), WT_TOMB_BUILDINSTR(WT_TOMB_OP_MSKBTM, 8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi16",
		MkTV(WTT_CHAR),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CCHARI32), WT_TOMB_BUILDINSTR(WT_TOMB_OP_MSKBTM, 16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou16",
		MkTV(WTT_CHAR),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CCHARU32), WT_TOMB_BUILDINSTR(WT_TOMB_OP_MSKBTM, 16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi32",
		MkTV(WTT_CHAR),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CCHARI32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou32",
		MkTV(WTT_CHAR),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CCHARU32)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi64",
		MkTV(WTT_CHAR),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CCHARI64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou64",
		MkTV(WTT_CHAR),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CCHARU64)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi8",
		MkTV(WTT_I8),
		MkTV(WTT_I8),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi8",
		MkTV(WTT_U8),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU8TOI8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi8",
		MkTV(WTT_I16),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI16TOI8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi8",
		MkTV(WTT_U16),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU16TOI8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi8",
		MkTV(WTT_I32),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32TOI8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi8",
		MkTV(WTT_U32),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32TOI8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi8",
		MkTV(WTT_I64),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI64TOI8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi8",
		MkTV(WTT_U64),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU64TOI8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou8",
		MkTV(WTT_I8),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI8TOU8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou8",
		MkTV(WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou8",
		MkTV(WTT_I16),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI16TOU8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou8",
		MkTV(WTT_U16),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU16TOU8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou8",
		MkTV(WTT_I32),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32TOU8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou8",
		MkTV(WTT_U32),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32TOU8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou8",
		MkTV(WTT_I64),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI64TOU8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou8",
		MkTV(WTT_U64),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU64TOU8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi16",
		MkTV(WTT_I8),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SGNXD, 8), WT_TOMB_BUILDINSTR(WT_TOMB_OP_MSKBTM, 16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi16",
		MkTV(WTT_U8),
		MkTV(WTT_I16),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi16",
		MkTV(WTT_I16),
		MkTV(WTT_I16),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi16",
		MkTV(WTT_U16),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU16TOI16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi16",
		MkTV(WTT_I32),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32TOI16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi16",
		MkTV(WTT_U32),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32TOI16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi16",
		MkTV(WTT_I64),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI64TOI16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("toi16",
		MkTV(WTT_U64),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU64TOI16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou16",
		MkTV(WTT_I8),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI8TOU8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou16",
		MkTV(WTT_U8),
		MkTV(WTT_U16),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou16",
		MkTV(WTT_I16),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI16TOU16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou16",
		MkTV(WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou16",
		MkTV(WTT_I32),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI32TOU16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou16",
		MkTV(WTT_U32),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU32TOU16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou16",
		MkTV(WTT_I64),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CI64TOU16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("tou16",
		MkTV(WTT_U64),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_CU64TOU16)),
		++mLastUniqueId));

	// handle

	mBuiltins.push_back(Builtin("nullhandle",
		MkTV(),
		MkTV(WTT_HANDLE),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_NULLHANDLE)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_HANDLE, WTT_HANDLE),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_HANDLEEQUAL)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_HANDLE, WTT_HANDLE),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_HANDLEEQUAL), WT_TOMB_BUILDINSTR(WT_TOMB_OP_LNOT, 0)),
		++mLastUniqueId));

	// boolean logic

	mBuiltins.push_back(Builtin("!",
		MkTV(WTT_BOOL),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_LNOT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("&&",
		MkTV(WTT_BOOL, WTT_BOOL),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_LAND, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("||",
		MkTV(WTT_BOOL, WTT_BOOL),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_LOR, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("^",
		MkTV(WTT_BOOL, WTT_BOOL),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_LXOR, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_BOOL, WTT_BOOL),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_LXOR, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_BOOL, WTT_BOOL),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_LNOT, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_LXOR, 0)),
		++mLastUniqueId));

	// integer operators: + - * / %

	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_I8, WTT_I8),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_ADDB_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_I8, WTT_I8),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SUBB_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("*",
		MkTV(WTT_I8, WTT_I8),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SMULB, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("/",
		MkTV(WTT_I8, WTT_I8),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SDIVB, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("%",
		MkTV(WTT_I8, WTT_I8),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SMODB, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_ADDB_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SUBB_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("*",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_MULB, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("/",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_DIVB, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("%",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_MODB, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_I16, WTT_I16),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_ADDH_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_I16, WTT_I16),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SUBH_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("*",
		MkTV(WTT_I16, WTT_I16),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SMULH, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("/",
		MkTV(WTT_I16, WTT_I16),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SDIVH, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("%",
		MkTV(WTT_I16, WTT_I16),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SMODH, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_ADDH_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SUBH_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("*",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_MULH, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("/",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_DIVH, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("%",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_MODH, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_I32, WTT_I32),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_ADD_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_I32, WTT_I32),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SUB_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("*",
		MkTV(WTT_I32, WTT_I32),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SMUL_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("/",
		MkTV(WTT_I32, WTT_I32),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SDIV_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("%",
		MkTV(WTT_I32, WTT_I32),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SMOD, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_U32, WTT_U32),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_ADD_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_U32, WTT_U32),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SUB_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("*",
		MkTV(WTT_U32, WTT_U32),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_MUL_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("/",
		MkTV(WTT_U32, WTT_U32),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_DIV_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("%",
		MkTV(WTT_U32, WTT_U32),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_MOD, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_I64, WTT_I64),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_ADDL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_I64, WTT_I64),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SUBL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("*",
		MkTV(WTT_I64, WTT_I64),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SMULL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("/",
		MkTV(WTT_I64, WTT_I64),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SDIVL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("%",
		MkTV(WTT_I64, WTT_I64),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SMODL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_U64, WTT_U64),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_ADDL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_U64, WTT_U64),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SUBL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("*",
		MkTV(WTT_U64, WTT_U64),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_MULL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("/",
		MkTV(WTT_U64, WTT_U64),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_DIVL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("%",
		MkTV(WTT_U64, WTT_U64),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_MODL, 0)),
		++mLastUniqueId));

	// unary integer operators: + -

	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_I8),
		MkTV(WTT_I8),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_I8),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_NEGB)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_NEG, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_MSKBTM, 8)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_I16),
		MkTV(WTT_I16),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_I16),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_NEGH)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_NEG, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_MSKBTM, 16)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_I32),
		MkTV(WTT_I32),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_I32),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_NEG, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_U32),
		MkTV(WTT_U32),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_U32),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_NEG, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_I64),
		MkTV(WTT_I64),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_I64),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_NEGL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_U64),
		MkTV(WTT_U64),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_U64),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_NEGL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FNEG, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FNEGL, 0)),
		++mLastUniqueId));

	// integer operators: << >> & | ^ ~

	const TypeVec bw8  = MkTV(WTT_I8,  WTT_U8);
	const TypeVec bw16 = MkTV(WTT_I16, WTT_U16);
	const TypeVec bw32 = MkTV(WTT_I32, WTT_U32);
	const TypeVec bw64 = MkTV(WTT_I64, WTT_U64);

	for(TypeVec::const_iterator it=bw8.begin(),end=bw8.end();it!=end;++it)
	{
		mBuiltins.push_back(Builtin("<<",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SHLB_S, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("&",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_BAND, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("|",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_BOR, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("^",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_XOR, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("~",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_BNOTB, 0)),
			++mLastUniqueId));
	}

	mBuiltins.push_back(Builtin(">>",
		MkTV(WTT_I8, WTT_I8),
		MkTV(WTT_I8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_SARB)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">>",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_U8),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SHR_S, 0)),
		++mLastUniqueId));

	for(TypeVec::const_iterator it=bw16.begin(),end=bw16.end();it!=end;++it)
	{
		mBuiltins.push_back(Builtin("<<",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SHLH_S, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("&",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_BAND, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("|",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_BOR, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("^",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_XOR, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("~",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_BNOTH, 0)),
			++mLastUniqueId));
	}

	mBuiltins.push_back(Builtin(">>",
		MkTV(WTT_I16, WTT_I16),
		MkTV(WTT_I16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_SARH)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">>",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_U16),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SHR_S, 0)),
		++mLastUniqueId));

	for(TypeVec::const_iterator it=bw32.begin(),end=bw32.end();it!=end;++it)
	{
		mBuiltins.push_back(Builtin("<<",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SHL_S, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("&",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_BAND, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("|",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_BOR, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("^",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_XOR, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("~",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_BNOT, 0)),
			++mLastUniqueId));
	}

	mBuiltins.push_back(Builtin(">>",
		MkTV(WTT_I32, WTT_I32),
		MkTV(WTT_I32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SAR_S, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">>",
		MkTV(WTT_U32, WTT_U32),
		MkTV(WTT_U32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SHR_S, 0)),
		++mLastUniqueId));

	for(TypeVec::const_iterator it=bw64.begin(),end=bw64.end();it!=end;++it)
	{
		mBuiltins.push_back(Builtin("<<",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SHLL, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("&",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_BANDL, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("|",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_BORL, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("^",
			MkTV(*it, *it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_XORL, 0)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("~",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_BNOTL, 0)),
			++mLastUniqueId));
	}

	mBuiltins.push_back(Builtin(">>",
		MkTV(WTT_I64, WTT_I64),
		MkTV(WTT_I64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SARL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">>",
		MkTV(WTT_U64, WTT_U64),
		MkTV(WTT_U64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SHRL, 0)),
		++mLastUniqueId));

	// integer operators: == != < <= > >=

	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_I32, WTT_I32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_I32, WTT_I32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<",
		MkTV(WTT_I32, WTT_I32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<=",
		MkTV(WTT_I32, WTT_I32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">",
		MkTV(WTT_I32, WTT_I32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">=",
		MkTV(WTT_I32, WTT_I32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_U32, WTT_U32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_U32, WTT_U32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<",
		MkTV(WTT_U32, WTT_U32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<=",
		MkTV(WTT_U32, WTT_U32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">",
		MkTV(WTT_U32, WTT_U32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">=",
		MkTV(WTT_U32, WTT_U32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_I64, WTT_I64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_I64, WTT_I64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<",
		MkTV(WTT_I64, WTT_I64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<=",
		MkTV(WTT_I64, WTT_I64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">",
		MkTV(WTT_I64, WTT_I64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">=",
		MkTV(WTT_I64, WTT_I64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_U64, WTT_U64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_U64, WTT_U64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<",
		MkTV(WTT_U64, WTT_U64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<=",
		MkTV(WTT_U64, WTT_U64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">",
		MkTV(WTT_U64, WTT_U64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">=",
		MkTV(WTT_U64, WTT_U64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_I16, WTT_I16),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPH, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_I16, WTT_I16),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPH, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<",
		MkTV(WTT_I16, WTT_I16),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPH, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<=",
		MkTV(WTT_I16, WTT_I16),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPH, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">",
		MkTV(WTT_I16, WTT_I16),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPH, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">=",
		MkTV(WTT_I16, WTT_I16),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPH, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<=",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">=",
		MkTV(WTT_U16, WTT_U16),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_I8, WTT_I8),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPB, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_I8, WTT_I8),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPB, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<",
		MkTV(WTT_I8, WTT_I8),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPB, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<=",
		MkTV(WTT_I8, WTT_I8),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPB, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">",
		MkTV(WTT_I8, WTT_I8),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPB, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">=",
		MkTV(WTT_I8, WTT_I8),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_SCMPB, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<=",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">=",
		MkTV(WTT_U8, WTT_U8),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_CMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGE, 0)),
		++mLastUniqueId));

	// float operators: + - * / %

	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_F32, WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FADD, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_F32, WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FSUB, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("*",
		MkTV(WTT_F32, WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FMUL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("/",
		MkTV(WTT_F32, WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FDIV, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("%",
		MkTV(WTT_F32, WTT_F32),
		MkTV(WTT_F32),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FMOD, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("+",
		MkTV(WTT_F64, WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FADDL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("-",
		MkTV(WTT_F64, WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FSUBL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("*",
		MkTV(WTT_F64, WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FMULL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("/",
		MkTV(WTT_F64, WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FDIVL, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("%",
		MkTV(WTT_F64, WTT_F64),
		MkTV(WTT_F64),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FMODL, 0)),
		++mLastUniqueId));

	// float operators: == != < <= > >=

	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_F32, WTT_F32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FCMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_F32, WTT_F32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FCMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<",
		MkTV(WTT_F32, WTT_F32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FCMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<=",
		MkTV(WTT_F32, WTT_F32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FCMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">",
		MkTV(WTT_F32, WTT_F32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FCMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">=",
		MkTV(WTT_F32, WTT_F32),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FCMP, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("==",
		MkTV(WTT_F64, WTT_F64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FCMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QEQ, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("!=",
		MkTV(WTT_F64, WTT_F64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FCMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QNE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<",
		MkTV(WTT_F64, WTT_F64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FCMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("<=",
		MkTV(WTT_F64, WTT_F64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FCMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QLE, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">",
		MkTV(WTT_F64, WTT_F64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FCMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGT, 0)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin(">=",
		MkTV(WTT_F64, WTT_F64),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_FCMPL, 0), WT_TOMB_BUILDINSTR(WT_TOMB_OP_QGE, 0)),
		++mLastUniqueId));

	// file operations

	mBuiltins.push_back(Builtin("access",
		MkTV(WTT_STRING, WTT_I32),
		MkTV(WTT_BOOL),
		false,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_FILEACCESS)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("stat",
		MkTV(WTT_STRING),
		MkTV(WTT_BOOL, WTT_CHAR, WTT_U64, WTT_U64, WTT_U64),
		false,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_FILESTAT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fopen",
		MkTV(WTT_STRING, WTT_CHAR),
		MkTV(WTT_FILE),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("feof",
		MkTV(WTT_FILE),
		MkTV(WTT_BOOL),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fstat",
		MkTV(WTT_FILE),
		MkTV(WTT_BOOL, WTT_CHAR, WTT_U64, WTT_U64, WTT_U64),
		false, true,
		MkIV(),
		++mLastUniqueId));

	const TypeVec seq_io_types = MkTV(
		WTT_I8, WTT_U8,
		WTT_I16, WTT_U16,
		WTT_I32, WTT_U32, WTT_F32,
		WTT_I64, WTT_U64, WTT_F64);

	for(TypeVec::const_iterator it=seq_io_types.begin(),end=seq_io_types.end();it!=end;++it)
	{
		mBuiltins.push_back(Builtin("fread",
			MkTV(WTT_FILE, WTTF_SEQ|(*it)),
			MkTV(WTT_U32),
			false, true,
			MkIV(),
			++mLastUniqueId));
	}

	mBuiltins.push_back(Builtin("freadchar",
		MkTV(WTT_FILE),
		MkTV(WTT_CHAR),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadbool",
		MkTV(WTT_FILE),
		MkTV(WTT_BOOL),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadstr",
		MkTV(WTT_FILE),
		MkTV(WTT_STRING),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadstr",
		MkTV(WTT_FILE, WTT_CHAR),
		MkTV(WTT_STRING),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadln",
		MkTV(WTT_FILE),
		MkTV(WTT_STRING),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadi8",
		MkTV(WTT_FILE),
		MkTV(WTT_I8),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadu8",
		MkTV(WTT_FILE),
		MkTV(WTT_U8),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadi16",
		MkTV(WTT_FILE),
		MkTV(WTT_I16),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadu16",
		MkTV(WTT_FILE),
		MkTV(WTT_U16),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadi32",
		MkTV(WTT_FILE),
		MkTV(WTT_I32),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadu32",
		MkTV(WTT_FILE),
		MkTV(WTT_U32),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadf32",
		MkTV(WTT_FILE),
		MkTV(WTT_F32),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadi64",
		MkTV(WTT_FILE),
		MkTV(WTT_I64),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadu64",
		MkTV(WTT_FILE),
		MkTV(WTT_U64),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("freadf64",
		MkTV(WTT_FILE),
		MkTV(WTT_F64),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwrite",
		MkTV(WTT_FILE, WTT_CHAR),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwrite",
		MkTV(WTT_FILE, WTT_BOOL),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwrite",
		MkTV(WTT_FILE, WTT_STRING),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwriteln",
		MkTV(WTT_FILE, WTT_STRING),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwrite",
		MkTV(WTT_FILE, WTT_I8),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwrite",
		MkTV(WTT_FILE, WTT_U8),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwrite",
		MkTV(WTT_FILE, WTT_I16),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwrite",
		MkTV(WTT_FILE, WTT_U16),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwrite",
		MkTV(WTT_FILE, WTT_I32),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwrite",
		MkTV(WTT_FILE, WTT_U32),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwrite",
		MkTV(WTT_FILE, WTT_F32),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwrite",
		MkTV(WTT_FILE, WTT_I64),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwrite",
		MkTV(WTT_FILE, WTT_U64),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("fwrite",
		MkTV(WTT_FILE, WTT_F64),
		MkTV(),
		false, true,
		MkIV(),
		++mLastUniqueId));

	for(TypeVec::const_iterator it=seq_io_types.begin(),end=seq_io_types.end();it!=end;++it)
	{
		mBuiltins.push_back(Builtin("fwrite",
			MkTV(WTT_FILE, WTTF_SEQ | (*it)),
			MkTV(),
			false, true,
			MkIV(),
			++mLastUniqueId));
	}

	mBuiltins.push_back(Builtin("opendir",
		MkTV(WTT_STRING),
		MkTV(WTT_DIR),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("readdir",
		MkTV(WTT_DIR),
		MkTV(WTT_BOOL),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("dirgetname",
		MkTV(WTT_DIR),
		MkTV(WTT_STRING),
		false, true,
		MkIV(),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("dirgettype",
		MkTV(WTT_DIR),
		MkTV(WTT_CHAR),
		false, true,
		MkIV(),
		++mLastUniqueId));

	// system

	mBuiltins.push_back(Builtin("rand",
		MkTV(),
		MkTV(WTT_U32),
		false,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_RAND)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("randf",
		MkTV(),
		MkTV(WTT_F32),
		false,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_RANDF)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("srand",
		MkTV(WTT_U32),
		MkTV(),
		false,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_SRAND)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("islittleendian",
		MkTV(),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISLITTLEENDIAN)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("isbigendian",
		MkTV(),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISBIGENDIAN)),
		++mLastUniqueId));

	TypeVec t16 = MkTV(WTT_I16, WTT_U16);
	TypeVec t32 = MkTV(WTT_I32, WTT_U32, WTT_F32);
	TypeVec t64 = MkTV(WTT_I64, WTT_U64, WTT_F64);

	for(TypeVec::const_iterator it=t16.begin(),end=t16.end();it!=end;++it)
	{
		mBuiltins.push_back(Builtin("ltob",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_SWAP16)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("btol",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_SWAP16)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("ltoh",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_LSWAP16)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("htol",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_LSWAP16)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("btoh",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_BSWAP16)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("htob",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_BSWAP16)),
			++mLastUniqueId));
	}

	for(TypeVec::const_iterator it=t32.begin(),end=t32.end();it!=end;++it)
	{
		mBuiltins.push_back(Builtin("ltob",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_SWAP32)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("btol",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_SWAP32)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("ltoh",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_LSWAP32)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("htol",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_LSWAP32)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("btoh",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_BSWAP32)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("htob",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_BSWAP32)),
			++mLastUniqueId));
	}

	for(TypeVec::const_iterator it=t64.begin(),end=t64.end();it!=end;++it)
	{
		mBuiltins.push_back(Builtin("ltob",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_SWAP64)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("btol",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_SWAP64)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("ltoh",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_LSWAP64)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("htol",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_LSWAP64)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("btoh",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_BSWAP64)),
			++mLastUniqueId));
		mBuiltins.push_back(Builtin("htob",
			MkTV(*it),
			MkTV(*it),
			true,
			MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_BSWAP64)),
			++mLastUniqueId));
	}

	// character classification

	mBuiltins.push_back(Builtin("iscntrl",
		MkTV(WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISCNTRL)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("isprint",
		MkTV(WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISPRINT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("isspace",
		MkTV(WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISSPACE)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("isblank",
		MkTV(WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISBLANK)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("isgraph",
		MkTV(WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISGRAPH)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("ispunct",
		MkTV(WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISPUNCT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("isalnum",
		MkTV(WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISALNUM)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("isalpha",
		MkTV(WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISALPHA)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("isupper",
		MkTV(WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISUPPER)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("islower",
		MkTV(WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISLOWER)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("isdigit",
		MkTV(WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISDIGIT)),
		++mLastUniqueId));
	mBuiltins.push_back(Builtin("isxdigit",
		MkTV(WTT_CHAR),
		MkTV(WTT_BOOL),
		true,
		MkIV(WT_TOMB_BUILDINSTR(WT_TOMB_OP_EXEC_I, WTMP_ISXDIGIT)),
		++mLastUniqueId));
}

const Builtins builtins;

} // builtin
} // wtparser
