//================================================================================================
// $Id: parser_pre.h 6309 2023-01-16 20:19:42Z aron $
//
// Project:
//		Welltype Parser
// Contents:
//		Parser preinclude
// Authors:
//		Aron Barath, 2014-2023
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#ifndef WTPARSER_PARSER_PRE_H
#define WTPARSER_PARSER_PRE_H

#include <string>
#include <deque>
#include "../wtruntime/wtruntime.h"
#include "ast/ast_program.h"
#include "ast/ast_entity.h"
#include "ast/ast_metadata.h"
#include "ast/ast_mainblock.h"
#include "ast/ast_constructorblock.h"
#include "ast/ast_destructorblock.h"
#include "ast/ast_declareblock.h"
#include "ast/ast_compound.h"
#include "ast/ast_exceptioncompound.h"
#include "ast/ast_finally.h"
#include "ast/ast_ehandler.h"
#include "ast/ast_ehandlers.h"
#include "ast/ast_type.h"
#include "ast/ast_typeinfo.h"
#include "ast/ast_basictype.h"
#include "ast/ast_seqtype.h"
#include "ast/ast_variabledeclaration.h"
#include "ast/ast_variabledeclfragment.h"
#include "ast/ast_variableaccess.h"
#include "ast/ast_elementaccess.h"
#include "ast/ast_expression.h"
#include "ast/ast_lvalue.h"
#include "ast/ast_simultaneoustype.h"
#include "ast/ast_simultaneousexpression.h"
#include "ast/ast_stringliteral.h"
#include "ast/ast_booleanliteral.h"
#include "ast/ast_characterliteral.h"
#include "ast/ast_numberliteral.h"
#include "ast/ast_assignment.h"
#include "ast/ast_functionsignature.h"
#include "ast/ast_functiondefinition.h"
#include "ast/ast_functionargument.h"
#include "ast/ast_functiondeclaration.h"
#include "ast/ast_procedurecall.h"
#include "ast/ast_declaration.h"
#include "ast/ast_recordfield.h"
#include "ast/ast_recordtype.h"
#include "ast/ast_recorddeclaration.h"
#include "ast/ast_exceptiondeclaration.h"
#include "ast/ast_precondition.h"
#include "ast/ast_postcondition.h"
#include "ast/ast_memberaccess.h"
#include "ast/ast_andthen.h"
#include "ast/ast_orelse.h"
#include "ast/ast_returnstatement.h"
#include "ast/ast_operatordeclaration.h"
#include "ast/ast_raisestatement.h"
#include "ast/ast_whilestatement.h"
#include "ast/ast_dowhilestatement.h"
#include "ast/ast_forstatement.h"
#include "ast/ast_foreachstatement.h"
#include "ast/ast_ifstatement.h"
#include "ast/ast_globalblock.h"
#include "ast/ast_assertstatement.h"
#include "ast/ast_seqliteral.h"
#include "ast/ast_fieldassign.h"
#include "ast/ast_recordliteral.h"
#include "ast/ast_datatype.h"
#include "ast/ast_datatypedeclaration.h"
#include "ast/ast_datatypecons.h"
#include "ast/ast_datatypeliteral.h"
#include "ast/ast_switchstatement.h"
#include "ast/ast_switchcase.h"
#include "ast/ast_datatypecasestatement.h"
#include "ast/ast_defaultstatement.h"
#include "ast/ast_functiontype.h"
#include "ast/ast_functiontypedeclaration.h"
#include "ast/ast_functionpointercall.h"
#include "ast/ast_makefunctionpointer.h"
#include "ast/ast_unresolvedfunctionname.h"
#include "ast/ast_enumerator.h"
#include "ast/ast_enumtype.h"
#include "ast/ast_enumtypedeclaration.h"
#include "ast/ast_enumcasestatement.h"
#include "ast/ast_enumliteral.h"
#include "ast/ast_recordupdate.h"

typedef std::deque<wtast::Entity*> entities_t;
typedef std::deque<wtast::EHandler*> ehandlerlist_t;
typedef std::deque<wtast::Statement*> statements_t;
typedef std::deque<wtast::Declaration*> declarations_t;
typedef std::deque<wtast::RecordField*> recordfields_t;
typedef std::deque<wtast::DataTypeCons*> datatypeconses_t;
typedef std::deque<wtast::FieldAssign*> fieldassigns_t;
typedef std::deque<wtast::VariableDeclaration*> variabledecls_t;
typedef std::deque<wtast::SwitchCase*> switchcases_t;
typedef std::deque<wtast::Enumerator*> enumerators_t;
typedef std::deque<std::string> identlist_t;

struct throptions_t
{
	throptions_t() : dss(0), css(0) { }

	wt_u32 dss;
	wt_u32 css;
};

#define CTL_RESERVED_TOKEN  0xFFEE00EEul
#define CTL_INCLUDE         0xFFCC0011ul
#define CTL_COMMENT         0xFFCC00CCul
#define CTL_ONCE            0xFFCC00DDul
#define CTL_EOF             0xFFCC00EFul
#define CTL_UTF8_BOM        0xFFEFBBBFul

#endif // WTPARSER_PARSER_PRE_H
